@@ -1,8 +1,8 @@
 #ifndef ISL_FLOW_H
 #define ISL_FLOW_H
 
-#include <isl/set.h>
-#include <isl/map.h>
+#include <isl/set_type.h>
+#include <isl/map_type.h>
 #include <isl/union_set.h>
 #include <isl/union_map.h>
 
@@ -27,6 +27,9 @@ __isl_give isl_access_info *isl_access_info_add_source(
 	__isl_take isl_access_info *acc, __isl_take isl_map *source,
 	int must, void *source_user);
 void isl_access_info_free(__isl_take isl_access_info *acc);
+
+isl_ctx *isl_access_info_get_ctx(__isl_keep isl_access_info *acc);
+
 __isl_give isl_flow *isl_access_info_compute_flow(__isl_take isl_access_info *acc);
 int isl_flow_foreach(__isl_keep isl_flow *deps,
 	int (*fn)(__isl_take isl_map *dep, int must, void *dep_user, void *user),
@@ -34,6 +37,8 @@ int isl_flow_foreach(__isl_keep isl_flow *deps,
 __isl_give isl_map *isl_flow_get_no_source(__isl_keep isl_flow *deps, int must);
 void isl_flow_free(__isl_take isl_flow *deps);
 
+isl_ctx *isl_flow_get_ctx(__isl_keep isl_flow *deps);
+
 int isl_union_map_compute_flow(__isl_take isl_union_map *sink,
 	__isl_take isl_union_map *must_source,
 	__isl_take isl_union_map *may_source,
