@@ -1,11 +1,11 @@
 #ifndef ISL_OBJ_H
 #define ISL_OBJ_H
 
-#include <isl/set.h>
-#include <isl/map.h>
+#include <isl/set_type.h>
+#include <isl/map_type.h>
 #include <isl/union_set.h>
 #include <isl/union_map.h>
-#include <isl/polynomial.h>
+#include <isl/polynomial_type.h>
 #include <isl/printer.h>
 
 #if defined(__cplusplus)
