@@ -8,11 +8,16 @@
  */
 
 #include <isl_ctx_private.h>
-#include <isl/dim.h>
+#include <isl/space.h>
 #include <isl/seq.h>
 #include <isl_mat_private.h>
 #include "isl_map_private.h"
-#include <isl_dim_private.h>
+#include <isl_space_private.h>
+
+isl_ctx *isl_mat_get_ctx(__isl_keep isl_mat *mat)
+{
+	return mat ? mat->ctx : NULL;
+}
 
 struct isl_mat *isl_mat_alloc(struct isl_ctx *ctx,
 	unsigned n_row, unsigned n_col)
@@ -107,7 +112,7 @@ error:
 	return NULL;
 }
 
-struct isl_mat *isl_mat_sub_alloc(struct isl_ctx *ctx, isl_int **row,
+__isl_give isl_mat *isl_mat_sub_alloc6(isl_ctx *ctx, isl_int **row,
 	unsigned first_row, unsigned n_row, unsigned first_col, unsigned n_col)
 {
 	int i;
@@ -134,6 +139,15 @@ error:
 	return NULL;
 }
 
+__isl_give isl_mat *isl_mat_sub_alloc(__isl_keep isl_mat *mat,
+	unsigned first_row, unsigned n_row, unsigned first_col, unsigned n_col)
+{
+	if (!mat)
+		return NULL;
+	return isl_mat_sub_alloc6(mat->ctx, mat->row, first_row, n_row,
+				  first_col, n_col);
+}
+
 void isl_mat_sub_copy(struct isl_ctx *ctx, isl_int **dst, isl_int **src,
 	unsigned n_row, unsigned dst_col, unsigned src_col, unsigned n_col)
 {
@@ -248,7 +262,26 @@ error:
 	return NULL;
 }
 
-struct isl_mat *isl_mat_identity(struct isl_ctx *ctx, unsigned n_row)
+__isl_give isl_mat *isl_mat_set_element_si(__isl_take isl_mat *mat,
+	int row, int col, int v)
+{
+	mat = isl_mat_cow(mat);
+	if (!mat)
+		return NULL;
+	if (row < 0 || row >= mat->n_row)
+		isl_die(mat->ctx, isl_error_invalid, "row out of range",
+			goto error);
+	if (col < 0 || col >= mat->n_col)
+		isl_die(mat->ctx, isl_error_invalid, "column out of range",
+			goto error);
+	isl_int_set_si(mat->row[row][col], v);
+	return mat;
+error:
+	isl_mat_free(mat);
+	return NULL;
+}
+
+__isl_give isl_mat *isl_mat_diag(isl_ctx *ctx, unsigned n_row, isl_int d)
 {
 	int i;
 	struct isl_mat *mat;
@@ -258,13 +291,20 @@ struct isl_mat *isl_mat_identity(struct isl_ctx *ctx, unsigned n_row)
 		return NULL;
 	for (i = 0; i < n_row; ++i) {
 		isl_seq_clr(mat->row[i], i);
-		isl_int_set_si(mat->row[i][i], 1);
+		isl_int_set(mat->row[i][i], d);
 		isl_seq_clr(mat->row[i]+i+1, n_row-(i+1));
 	}
 
 	return mat;
 }
 
+__isl_give isl_mat *isl_mat_identity(isl_ctx *ctx, unsigned n_row)
+{
+	if (!ctx)
+		return NULL;
+	return isl_mat_diag(ctx, n_row, ctx->one);
+}
+
 struct isl_vec *isl_mat_vec_product(struct isl_mat *mat, struct isl_vec *vec)
 {
 	int i;
@@ -528,6 +568,7 @@ error:
 		isl_mat_free(*U);
 		*U = NULL;
 	}
+	isl_mat_free(M);
 	return NULL;
 }
 
@@ -937,6 +978,8 @@ struct isl_mat *isl_mat_product(struct isl_mat *left, struct isl_mat *right)
 	if (left->n_col == 0) {
 		for (i = 0; i < prod->n_row; ++i)
 			isl_seq_clr(prod->row[i], prod->n_col);
+		isl_mat_free(left);
+		isl_mat_free(right);
 		return prod;
 	}
 	for (i = 0; i < prod->n_row; ++i) {
@@ -982,7 +1025,7 @@ static int preimage(struct isl_ctx *ctx, isl_int **q, unsigned n,
 	if (has_div)
 		for (i = 0; i < n; ++i)
 			isl_int_mul(q[i][0], q[i][0], mat->row[0][0]);
-	t = isl_mat_sub_alloc(mat->ctx, q, 0, n, has_div, mat->n_row);
+	t = isl_mat_sub_alloc6(mat->ctx, q, 0, n, has_div, mat->n_row);
 	t = isl_mat_product(t, mat);
 	if (!t)
 		return -1;
@@ -1028,7 +1071,7 @@ struct isl_basic_set *isl_basic_set_preimage(struct isl_basic_set *bset,
 		if (!bset)
 			goto error;
 	} else if (mat->n_col < mat->n_row) {
-		bset->dim = isl_dim_cow(bset->dim);
+		bset->dim = isl_space_cow(bset->dim);
 		if (!bset->dim)
 			goto error;
 		bset->dim->n_out -= mat->n_row - mat->n_col;
@@ -1079,7 +1122,7 @@ struct isl_set *isl_set_preimage(struct isl_set *set, struct isl_mat *mat)
 			goto error;
 	}
 	if (mat->n_col != mat->n_row) {
-		set->dim = isl_dim_cow(set->dim);
+		set->dim = isl_space_cow(set->dim);
 		if (!set->dim)
 			goto error;
 		set->dim->n_out += mat->n_col;
@@ -1094,7 +1137,74 @@ error:
 	return NULL;
 }
 
-void isl_mat_dump(struct isl_mat *mat, FILE *out, int indent)
+/* Replace the variables x starting at pos in the rows q
+ * by x' with x = M x' with M the matrix mat.
+ * That is, replace the corresponding coefficients c by c M.
+ */
+static int transform(isl_ctx *ctx, isl_int **q, unsigned n,
+	unsigned pos, __isl_take isl_mat *mat)
+{
+	int i;
+	isl_mat *t;
+
+	t = isl_mat_sub_alloc6(ctx, q, 0, n, pos, mat->n_row);
+	t = isl_mat_product(t, mat);
+	if (!t)
+		return -1;
+	for (i = 0; i < n; ++i)
+		isl_seq_swp_or_cpy(q[i] + pos, t->row[i], t->n_col);
+	isl_mat_free(t);
+	return 0;
+}
+
+/* Replace the variables x of type "type" starting at "first" in "bset"
+ * by x' with x = M x' with M the matrix trans.
+ * That is, replace the corresponding coefficients c by c M.
+ *
+ * The transformation matrix should be a square matrix.
+ */
+__isl_give isl_basic_set *isl_basic_set_transform_dims(
+	__isl_take isl_basic_set *bset, enum isl_dim_type type, unsigned first,
+	__isl_take isl_mat *trans)
+{
+	isl_ctx *ctx;
+	unsigned pos;
+
+	bset = isl_basic_set_cow(bset);
+	if (!bset || !trans)
+		goto error;
+
+	ctx = isl_basic_set_get_ctx(bset);
+	if (trans->n_row != trans->n_col)
+		isl_die(trans->ctx, isl_error_invalid,
+			"expecting square transformation matrix", goto error);
+	if (first + trans->n_row > isl_basic_set_dim(bset, type))
+		isl_die(trans->ctx, isl_error_invalid,
+			"oversized transformation matrix", goto error);
+
+	pos = isl_basic_set_offset(bset, type) + first;
+
+	if (transform(ctx, bset->eq, bset->n_eq, pos, isl_mat_copy(trans)) < 0)
+		goto error;
+	if (transform(ctx, bset->ineq, bset->n_ineq, pos,
+		      isl_mat_copy(trans)) < 0)
+		goto error;
+	if (transform(ctx, bset->div, bset->n_div, 1 + pos,
+		      isl_mat_copy(trans)) < 0)
+		goto error;
+
+	ISL_F_CLR(bset, ISL_BASIC_SET_NORMALIZED);
+	ISL_F_CLR(bset, ISL_BASIC_SET_NORMALIZED_DIVS);
+
+	isl_mat_free(trans);
+	return bset;
+error:
+	isl_mat_free(trans);
+	isl_basic_set_free(bset);
+	return NULL;
+}
+
+void isl_mat_print_internal(__isl_keep isl_mat *mat, FILE *out, int indent)
 {
 	int i, j;
 
@@ -1123,6 +1233,11 @@ void isl_mat_dump(struct isl_mat *mat, FILE *out, int indent)
 	}
 }
 
+void isl_mat_dump(__isl_keep isl_mat *mat)
+{
+	isl_mat_print_internal(mat, stderr, 0);
+}
+
 struct isl_mat *isl_mat_drop_cols(struct isl_mat *mat, unsigned col, unsigned n)
 {
 	int r;
@@ -1238,6 +1353,29 @@ __isl_give isl_mat *isl_mat_add_rows(__isl_take isl_mat *mat, unsigned n)
 	return isl_mat_insert_rows(mat, mat->n_row, n);
 }
 
+__isl_give isl_mat *isl_mat_insert_zero_rows(__isl_take isl_mat *mat,
+	unsigned row, unsigned n)
+{
+	int i;
+
+	mat = isl_mat_insert_rows(mat, row, n);
+	if (!mat)
+		return NULL;
+	
+	for (i = 0; i < n; ++i)
+		isl_seq_clr(mat->row[row + i], mat->n_col);
+
+	return mat;
+}
+
+__isl_give isl_mat *isl_mat_add_zero_rows(__isl_take isl_mat *mat, unsigned n)
+{
+	if (!mat)
+		return NULL;
+
+	return isl_mat_insert_zero_rows(mat, mat->n_row, n);
+}
+
 void isl_mat_col_submul(struct isl_mat *mat,
 			int dst_col, isl_int f, int src_col)
 {
@@ -1438,6 +1576,10 @@ __isl_give isl_mat *isl_mat_scale_down(__isl_take isl_mat *mat, isl_int m)
 {
 	int i;
 
+	if (isl_int_is_one(m))
+		return mat;
+
+	mat = isl_mat_cow(mat);
 	if (!mat)
 		return NULL;
 
@@ -1461,3 +1603,30 @@ __isl_give isl_mat *isl_mat_normalize(__isl_take isl_mat *mat)
 
 	return mat;
 }
+
+__isl_give isl_mat *isl_mat_normalize_row(__isl_take isl_mat *mat, int row)
+{
+	mat = isl_mat_cow(mat);
+	if (!mat)
+		return NULL;
+
+	isl_seq_normalize(mat->ctx, mat->row[row], mat->n_col);
+
+	return mat;
+}
+
+/* Number of initial non-zero columns.
+ */
+int isl_mat_initial_non_zero_cols(__isl_keep isl_mat *mat)
+{
+	int i;
+
+	if (!mat)
+		return -1;
+
+	for (i = 0; i < mat->n_col; ++i)
+		if (row_first_non_zero(mat->row, mat->n_row, i) < 0)
+			break;
+
+	return i;
+}
