@@ -3,7 +3,7 @@
 
 struct isl_union_map {
 	int ref;
-	isl_dim *dim;
+	isl_space *dim;
 
 	struct isl_hash_table	table;
 };
