@@ -450,20 +450,22 @@ static struct clast_expr *div_expr(CloogConstraint *constraint, int pos,
 	cloog_int_t c;
 	struct clast_reduction *r;
 	struct clast_expr *e = NULL;
-	struct isl_div *div;
+	isl_aff *div;
 
-	div = isl_constraint_div(cloog_constraint_to_isl(constraint), pos);
+	div = isl_constraint_get_div(cloog_constraint_to_isl(constraint), pos);
 
 	cloog_int_init(c);
 	for (i = 0, nb_elts = 0; i < dim; ++i) {
 		struct cloog_isl_dim dim;
 
 		dim = constraint_cloog_dim_to_isl_dim(constraint, i);
-		isl_div_get_coefficient(div, dim.type, dim.pos, &c);
+		if (dim.type == isl_dim_set)
+			dim.type = isl_dim_in;
+		isl_aff_get_coefficient(div, dim.type, dim.pos, &c);
 		if (!cloog_int_is_zero(c))
 			++nb_elts;
 	}
-	isl_div_get_constant(div, &c);
+	isl_aff_get_constant(div, &c);
 	if (!cloog_int_is_zero(c))
 		++nb_elts;
 
@@ -473,7 +475,9 @@ static struct clast_expr *div_expr(CloogConstraint *constraint, int pos,
 		struct cloog_isl_dim dim;
 
 		dim = constraint_cloog_dim_to_isl_dim(constraint, i);
-		isl_div_get_coefficient(div, dim.type, dim.pos, &c);
+		if (dim.type == isl_dim_set)
+			dim.type = isl_dim_in;
+		isl_aff_get_coefficient(div, dim.type, dim.pos, &c);
 		if (cloog_int_is_zero(c))
 			continue;
 
@@ -481,16 +485,16 @@ static struct clast_expr *div_expr(CloogConstraint *constraint, int pos,
 
 		r->elts[nb_elts++] = &new_clast_term(c, v)->expr;
 	}
-	isl_div_get_constant(div, &c);
+	isl_aff_get_constant(div, &c);
 	if (!cloog_int_is_zero(c))
 		r->elts[nb_elts++] = &new_clast_term(c, NULL)->expr;
 
-	isl_div_get_denominator(div, &c);
+	isl_aff_get_denominator(div, &c);
 	e = &new_clast_binary(clast_bin_fdiv, &r->expr, c)->expr;
 
 	cloog_int_clear(c);
 
-	isl_div_free(div);
+	isl_aff_free(div);
 
 	return e;
 }
@@ -763,21 +767,17 @@ CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 	int level, CloogEqualities *equal, int nb_par, cloog_int_t *bound)
 {
 	int j;
-	isl_ctx *ctx;
-	isl_dim *idim;
+	isl_space *idim;
 	struct isl_basic_set *eq;
 	struct isl_basic_map *id;
 	struct cloog_isl_dim dim;
 	struct isl_constraint *c;
-	struct isl_div *div;
 	unsigned constraints_dim;
 	unsigned n_div;
 	isl_basic_set *bset, *orig;
-	isl_aff *aff;
 
 	bset = cloog_constraints_set_to_isl(constraints);
 	orig = isl_basic_set_copy(bset);
-	ctx = isl_basic_set_get_ctx(bset);
 	dim = set_cloog_dim_to_isl_dim(constraints, level - 1);
 	assert(dim.type == isl_dim_set);
 
@@ -797,7 +797,7 @@ CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 		return cloog_constraint_set_from_isl_basic_set(bset);
 	}
 
-	idim = isl_dim_map_from_set(isl_basic_set_get_dim(bset));
+	idim = isl_space_map_from_set(isl_basic_set_get_space(bset));
 	id = isl_basic_map_identity(idim);
 	id = isl_basic_map_remove_dims(id, isl_dim_out, dim.pos, 1);
 	bset = isl_basic_set_apply(bset, isl_basic_map_copy(id));
@@ -817,10 +817,9 @@ CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 		return cloog_constraint_set_from_isl_basic_set(bset);
 	}
 
-	div = isl_basic_set_div(isl_basic_set_copy(bset), 0);
-	aff = isl_aff_from_div(div);
-	aff = isl_aff_add_coefficient_si(aff, isl_dim_set, dim.pos, -1);
-	c = isl_equality_from_aff(aff);
+	c = isl_equality_alloc(isl_basic_set_get_local_space(bset));
+	c = isl_constraint_set_coefficient_si(c, isl_dim_div, 0, 1);
+	c = isl_constraint_set_coefficient_si(c, isl_dim_set, dim.pos, -1);
 	bset = isl_basic_set_add_constraint(bset, c);
 
 	isl_int_set_si(*bound, 0);
@@ -894,9 +893,9 @@ static isl_aff *extract_stride_offset(__isl_keep isl_constraint *c,
 	int level, CloogStride *stride)
 {
 	int i;
-	isl_dim *dim = isl_constraint_get_dim(c);
-	isl_local_space *ls = isl_local_space_from_dim(dim);
-	isl_aff *offset = isl_aff_zero(ls);
+	isl_space *dim = isl_constraint_get_space(c);
+	isl_local_space *ls = isl_local_space_from_space(dim);
+	isl_aff *offset = isl_aff_zero_on_domain(ls);
 	isl_int u;
 	unsigned nparam, nvar;
 
@@ -915,7 +914,7 @@ static isl_aff *extract_stride_offset(__isl_keep isl_constraint *c,
 			continue;
 		isl_constraint_get_coefficient(c, isl_dim_set, i, &u);
 		isl_int_mul(u, u, stride->factor);
-		offset = isl_aff_set_coefficient(offset, isl_dim_set, i, u);
+		offset = isl_aff_set_coefficient(offset, isl_dim_in, i, u);
 	}
 	isl_constraint_get_constant(c, &u);
 	isl_int_mul(u, u, stride->factor);
@@ -959,7 +958,7 @@ CloogConstraint *cloog_constraint_stride_lower_bound(CloogConstraint *c,
 	lower = isl_aff_scale(lower, stride->stride);
 	lower = isl_aff_add(lower, offset);
 	lower = isl_aff_neg(lower);
-	lower = isl_aff_add_coefficient_si(lower, isl_dim_set, level - 1, 1);
+	lower = isl_aff_add_coefficient_si(lower, isl_dim_in, level - 1, 1);
 
 	bound = isl_inequality_from_aff(lower);
 
