@@ -6,12 +6,18 @@
 #include <cloog/isl/cloog.h>
 #include <isl/list.h>
 #include <isl/constraint.h>
-#include <isl/div.h>
 #include <isl/ilp.h>
 #include <isl/aff.h>
 
+#ifdef OSL_SUPPORT
+#include <osl/macros.h>
+#include <osl/relation.h>
+#endif
+
 CloogDomain *cloog_domain_from_isl_set(struct isl_set *set)
 {
+	if (isl_set_is_params(set))
+		set = isl_set_from_params(set);
 	set = isl_set_detect_equalities(set);
 	set = isl_set_compute_divs(set);
 	return (CloogDomain *)set;
@@ -400,7 +406,7 @@ CloogDomain *cloog_domain_read_context(CloogState *state, FILE *input)
 	struct isl_ctx *ctx = state->backend->ctx;
 	isl_set *set;
 
-	set = isl_set_read_from_file(ctx, input, 0);
+	set = isl_set_read_from_file(ctx, input);
 	set = isl_set_move_dims(set, isl_dim_param, 0,
 				isl_dim_set, 0, isl_set_dim(set, isl_dim_set));
 
@@ -434,7 +440,12 @@ CloogDomain *cloog_domain_union_read(CloogState *state,
 	struct isl_ctx *ctx = state->backend->ctx;
 	struct isl_set *set;
 
-	set = isl_set_read_from_file(ctx, input, nb_parameters);
+	set = isl_set_read_from_file(ctx, input);
+	if (isl_set_dim(set, isl_dim_param) != nb_parameters) {
+		int dim = isl_set_dim(set, isl_dim_set);
+		set = isl_set_move_dims(set, isl_dim_param, 0,
+			    isl_dim_set, dim - nb_parameters, nb_parameters);
+	}
 	return cloog_domain_from_isl_set(set);
 }
 
@@ -461,7 +472,12 @@ CloogScattering *cloog_domain_read_scattering(CloogDomain *domain, FILE *input)
 
 	dim = isl_set_dim(set, isl_dim_set);
 	nparam = isl_set_dim(set, isl_dim_param);
-	scat = isl_map_read_from_file(ctx, input, nparam);
+	scat = isl_map_read_from_file(ctx, input);
+	if (isl_map_dim(scat, isl_dim_param) != nparam) {
+		int n_out = isl_map_dim(scat, isl_dim_out);
+		scat = isl_map_move_dims(scat, isl_dim_param, 0,
+					isl_dim_out, n_out - nparam, nparam);
+	}
 	if (isl_map_dim(scat, isl_dim_in) != dim) {
 		n_scat = isl_map_dim(scat, isl_dim_out) - dim;
 		scat = isl_map_move_dims(scat, isl_dim_in, 0,
@@ -480,17 +496,18 @@ CloogScattering *cloog_domain_read_scattering(CloogDomain *domain, FILE *input)
  * Returns a pointer to the constraint if successful; NULL otherwise.
  */
 static struct isl_constraint *isl_constraint_read_from_matrix(
-	struct isl_dim *dim, cloog_int_t *row)
+	struct isl_space *dim, cloog_int_t *row)
 {
 	struct isl_constraint *constraint;
 	int j;
-	int nvariables = isl_dim_size(dim, isl_dim_set);
-	int nparam = isl_dim_size(dim, isl_dim_param);
+	int nvariables = isl_space_dim(dim, isl_dim_set);
+	int nparam = isl_space_dim(dim, isl_dim_param);
+	isl_local_space *ls = isl_local_space_from_space(dim);
 
 	if (cloog_int_is_zero(row[0]))
-		constraint = isl_equality_alloc(dim);
+		constraint = isl_equality_alloc(ls);
 	else
-		constraint = isl_inequality_alloc(dim);
+		constraint = isl_inequality_alloc(ls);
 
 	for (j = 0; j < nvariables; ++j)
 		isl_constraint_set_coefficient(constraint, isl_dim_out, j,
@@ -513,7 +530,7 @@ static struct isl_constraint *isl_constraint_read_from_matrix(
 static struct isl_basic_set *isl_basic_set_read_from_matrix(struct isl_ctx *ctx,
 	CloogMatrix* matrix, int nparam)
 {
-	struct isl_dim *dim;
+	struct isl_space *dim;
 	struct isl_basic_set *bset;
 	int i;
 	unsigned nrows, ncolumns;
@@ -522,18 +539,18 @@ static struct isl_basic_set *isl_basic_set_read_from_matrix(struct isl_ctx *ctx,
 	ncolumns = matrix->NbColumns;
 	int nvariables = ncolumns - 2 - nparam;
 
-	dim = isl_dim_set_alloc(ctx, nparam, nvariables);
+	dim = isl_space_set_alloc(ctx, nparam, nvariables);
 
-	bset = isl_basic_set_universe(isl_dim_copy(dim));
+	bset = isl_basic_set_universe(isl_space_copy(dim));
 
 	for (i = 0; i < nrows; ++i) {
 		cloog_int_t *row = matrix->p[i];
 		struct isl_constraint *constraint =
-			isl_constraint_read_from_matrix(isl_dim_copy(dim), row);
+			isl_constraint_read_from_matrix(isl_space_copy(dim), row);
 		bset = isl_basic_set_add_constraint(bset, constraint);
 	}
 
-	isl_dim_free(dim);
+	isl_space_free(dim);
 
 	return bset;
 }
@@ -567,12 +584,12 @@ CloogScattering *cloog_scattering_from_cloog_matrix(CloogState *state,
 	struct isl_ctx *ctx = state->backend->ctx;
 	struct isl_basic_set *bset;
 	struct isl_basic_map *scat;
-	struct isl_dim *dims;
+	struct isl_space *dims;
 	unsigned dim;
 
 	bset = isl_basic_set_read_from_matrix(ctx, matrix, nb_par);
 	dim = isl_basic_set_n_dim(bset) - nb_scat;
-	dims = isl_dim_alloc(ctx, nb_par, nb_scat, dim);
+	dims = isl_space_alloc(ctx, nb_par, nb_scat, dim);
 
 	scat = isl_basic_map_from_basic_set(bset, dims);
 	scat = isl_basic_map_reverse(scat);
@@ -585,6 +602,65 @@ CloogScattering *cloog_scattering_from_cloog_matrix(CloogState *state,
  ******************************************************************************/
 
 
+#ifdef OSL_SUPPORT
+/**
+ * Converts an openscop relation to a CLooG domain.
+ * \param[in,out] state    CLooG state.
+ * \param[in]     relation OpenScop relation to convert.
+ * \return A new CloogDomain corresponding to the input OpenScop relation.
+ */
+CloogDomain *cloog_domain_from_osl_relation(CloogState *state,
+                                            osl_relation_p relation) {
+  char *str;
+  struct isl_ctx *ctx = state->backend->ctx;
+  isl_set *set;
+  CloogDomain *domain = NULL;
+
+  if (relation != NULL) {
+    if (relation->precision != OSL_PRECISION_MP)
+      cloog_die("Non-GMP precision is not supported yet.\n");
+
+    str = osl_relation_spprint_polylib(relation, NULL);
+    set = isl_set_read_from_str(ctx, str);
+    free(str);
+
+    domain = cloog_domain_from_isl_set(set);
+  }
+
+  return domain;
+}
+
+
+/**
+ * Converts an openscop scattering relation to a CLooG scattering.
+ * \param[in,out] state    CLooG state.
+ * \param[in]     relation OpenScop relation to convert.
+ * \return A new CloogScattering corresponding to the input OpenScop relation.
+ */
+CloogScattering *cloog_scattering_from_osl_relation(CloogState *state,
+                                                    osl_relation_p relation) {
+  char *str;
+  struct isl_ctx *ctx = state->backend->ctx;
+  isl_map *map;
+  CloogScattering *scattering = NULL;
+
+  if (relation != NULL) {
+    if (relation->precision != OSL_PRECISION_MP)
+      cloog_die("Non-GMP precision is not supported yet.\n");
+
+    if (relation->type != OSL_TYPE_SCATTERING)
+      cloog_die("Cannot convert a non-scattering relation to a scattering.\n");
+
+    str = osl_relation_spprint_polylib(relation, NULL);
+    map = isl_map_read_from_str(ctx, str);
+    free(str);
+
+    scattering = cloog_scattering_from_isl_map(map);
+  }
+
+  return scattering;
+}
+#endif
 
 /**
  * cloog_domain_isempty function:
@@ -602,10 +678,10 @@ int cloog_domain_isempty(CloogDomain *domain)
  */
 CloogDomain *cloog_domain_universe(CloogState *state, unsigned dim)
 {
-	struct isl_dim *dims;
+	struct isl_space *dims;
 	struct isl_basic_set *bset;
 
-	dims = isl_dim_set_alloc(state->backend->ctx, 0, dim);
+	dims = isl_space_set_alloc(state->backend->ctx, 0, dim);
 	bset = isl_basic_set_universe(dims);
 	return cloog_domain_from_isl_set(isl_set_from_basic_set(bset));
 }
@@ -638,7 +714,8 @@ CloogDomain *cloog_domain_project(CloogDomain *domain, int level)
 CloogDomain *cloog_domain_extend(CloogDomain *domain, int dim)
 {
 	isl_set *set = isl_set_from_cloog_domain(domain);
-	set = isl_set_extend(isl_set_copy(set), isl_set_n_param(set), dim);
+	int n = isl_set_dim(set, isl_dim_set);
+	set = isl_set_add_dims(isl_set_copy(set), isl_dim_set, dim - n);
 	return cloog_domain_from_isl_set(set);
 }
 
@@ -852,7 +929,7 @@ static int constraint_stride_lower(__isl_take isl_constraint *c, void *user)
 	b = isl_aff_scale(b, csl->stride->stride);
 	isl_int_neg(v, csl->stride->offset);
 	b = isl_aff_add_constant(b, v);
-	b = isl_aff_add_coefficient_si(b, isl_dim_set, csl->level - 1, 1);
+	b = isl_aff_add_coefficient_si(b, isl_dim_in, csl->level - 1, 1);
 
 	bound = isl_inequality_from_aff(b);
 
@@ -905,7 +982,7 @@ static int constraint_stride_lower_c(__isl_take isl_constraint *c, void *user)
 
 	d = isl_constraint_get_aff(csl_c);
 	d = isl_aff_drop_dims(d, isl_dim_div, 0, isl_aff_dim(d, isl_dim_div));
-	d = isl_aff_set_coefficient_si(d, isl_dim_set, csl->level - 1, 0);
+	d = isl_aff_set_coefficient_si(d, isl_dim_in, csl->level - 1, 0);
 	d = isl_aff_scale(d, csl->stride->factor);
 
 	b = isl_constraint_get_bound(c, isl_dim_set, csl->level - 1);
@@ -916,7 +993,7 @@ static int constraint_stride_lower_c(__isl_take isl_constraint *c, void *user)
 	b = isl_aff_floor(b);
 	b = isl_aff_scale(b, csl->stride->stride);
 	b = isl_aff_sub(b, d);
-	b = isl_aff_add_coefficient_si(b, isl_dim_set, csl->level - 1, 1);
+	b = isl_aff_add_coefficient_si(b, isl_dim_in, csl->level - 1, 1);
 
 	bound = isl_inequality_from_aff(b);
 
@@ -1128,7 +1205,7 @@ int cloog_scattering_lazy_block(CloogScattering *s1, CloogScattering *s2,
 			    CloogScatteringList *scattering, int scattdims)
 {
 	int i;
-	struct isl_dim *dim;
+	struct isl_space *dim;
 	struct isl_map *rel;
 	struct isl_set *delta;
 	isl_map *map1 = isl_map_from_cloog_scattering(s1);
@@ -1141,8 +1218,8 @@ int cloog_scattering_lazy_block(CloogScattering *s1, CloogScattering *s2,
 	if (n_scat != isl_map_dim(map2, isl_dim_out))
 		return 0;
 
-	dim = isl_map_get_dim(map1);
-	dim = isl_dim_map_from_set(isl_dim_domain(dim));
+	dim = isl_map_get_space(map1);
+	dim = isl_space_map_from_set(isl_space_domain(dim));
 	rel = isl_map_identity(dim);
 	rel = isl_map_apply_domain(rel, isl_map_copy(map1));
 	rel = isl_map_apply_range(rel, isl_map_copy(map2));
@@ -1345,18 +1422,18 @@ CloogDomain *cloog_domain_scatter(CloogDomain *domain, CloogScattering *scatt)
 
 static int add_domain_from_map(__isl_take isl_map *map, void *user)
 {
-	isl_dim *dim;
+	isl_space *dim;
 	const char *name;
 	CloogDomain *domain;
 	CloogScattering *scat;
 	CloogUnionDomain **ud = (CloogUnionDomain **)user;
 
-	dim = isl_map_get_dim(map);
-	name = isl_dim_get_tuple_name(dim, isl_dim_in);
+	dim = isl_map_get_space(map);
+	name = isl_space_get_tuple_name(dim, isl_dim_in);
 	domain = cloog_domain_from_isl_set(isl_map_domain(isl_map_copy(map)));
 	scat = cloog_scattering_from_isl_map(map);
 	*ud = cloog_union_domain_add_domain(*ud, name, domain, scat, NULL);
-	isl_dim_free(dim);
+	isl_space_free(dim);
 
 	return 0;
 }
@@ -1376,19 +1453,19 @@ CloogUnionDomain *cloog_union_domain_from_isl_union_map(
 {
 	int i;
 	int nparam;
-	isl_dim *dim;
+	isl_space *dim;
 	CloogUnionDomain *ud;
 
-	dim = isl_union_map_get_dim(umap);
-	nparam = isl_dim_size(dim, isl_dim_param);
+	dim = isl_union_map_get_space(umap);
+	nparam = isl_space_dim(dim, isl_dim_param);
 
 	ud = cloog_union_domain_alloc(nparam);
 
 	for (i = 0; i < nparam; ++i) {
-		const char *s = isl_dim_get_name(dim, isl_dim_param, i);
+		const char *s = isl_space_get_dim_name(dim, isl_dim_param, i);
 		ud = cloog_union_domain_set_name(ud, CLOOG_PARAM, i, s);
 	}
-	isl_dim_free(dim);
+	isl_space_free(dim);
 
 	if (isl_union_map_foreach_map(umap, &add_domain_from_map, &ud) < 0) {
 		isl_union_map_free(umap);
@@ -1401,7 +1478,7 @@ CloogUnionDomain *cloog_union_domain_from_isl_union_map(
 	return ud;
 }
 
-static int count_same_name(__isl_keep isl_dim *dim,
+static int count_same_name(__isl_keep isl_space *dim,
 	enum isl_dim_type type, unsigned pos, const char *name)
 {
 	enum isl_dim_type t;
@@ -1410,9 +1487,9 @@ static int count_same_name(__isl_keep isl_dim *dim,
 	int len = strlen(name);
 
 	for (t = isl_dim_param; t <= type && t <= isl_dim_out; ++t) {
-		s = t == type ? pos : isl_dim_size(dim, t);
+		s = t == type ? pos : isl_space_dim(dim, t);
 		for (p = 0; p < s; ++p) {
-			const char *n = isl_dim_get_name(dim, t, p);
+			const char *n = isl_space_get_dim_name(dim, t, p);
 			if (n && !strncmp(n, name, len))
 				count++;
 		}
@@ -1420,30 +1497,29 @@ static int count_same_name(__isl_keep isl_dim *dim,
 	return count;
 }
 
-static int add_domain(__isl_take isl_set *set, void *user)
+static CloogUnionDomain *add_domain(__isl_take isl_set *set, CloogUnionDomain *ud)
 {
 	int i, nvar;
 	isl_ctx *ctx;
-	isl_dim *dim;
+	isl_space *dim;
 	char buffer[20];
 	const char *name;
 	CloogDomain *domain;
-	CloogUnionDomain **ud = (CloogUnionDomain **)user;
 
 	ctx = isl_set_get_ctx(set);
-	dim = isl_set_get_dim(set);
-	name = isl_dim_get_tuple_name(dim, isl_dim_set);
+	dim = isl_set_get_space(set);
+	name = isl_space_get_tuple_name(dim, isl_dim_set);
 	set = isl_set_flatten(set);
 	set = isl_set_set_tuple_name(set, NULL);
 	domain = cloog_domain_from_isl_set(set);
-	*ud = cloog_union_domain_add_domain(*ud, name, domain, NULL, NULL);
+	ud = cloog_union_domain_add_domain(ud, name, domain, NULL, NULL);
 
-	nvar = isl_dim_size(dim, isl_dim_set);
+	nvar = isl_space_dim(dim, isl_dim_set);
 	for (i = 0; i < nvar; ++i) {
 		char *long_name = NULL;
 		int n;
 
-		name = isl_dim_get_name(dim, isl_dim_set, i);
+		name = isl_space_get_dim_name(dim, isl_dim_set, i);
 		if (!name) {
 			snprintf(buffer, sizeof(buffer), "i%d", i);
 			name = buffer;
@@ -1457,46 +1533,40 @@ static int add_domain(__isl_take isl_set *set, void *user)
 			snprintf(long_name, size, "%s_%d", name, n);
 			name = long_name;
 		}
-		*ud = cloog_union_domain_set_name(*ud, CLOOG_ITER, i, name);
+		ud = cloog_union_domain_set_name(ud, CLOOG_ITER, i, name);
 		free(long_name);
 	}
-	isl_dim_free(dim);
+	isl_space_free(dim);
 
-	return 0;
+	return ud;
 }
 
 /**
- * Construct a CloogUnionDomain from an isl_union_set.
+ * Construct a CloogUnionDomain from an isl_set.
  * The statement names are set to the names of the
  * spaces.  The parameter and iterator names of the result are set to those of
  * the input, but the scattering dimension names are left unspecified.
  */
-CloogUnionDomain *cloog_union_domain_from_isl_union_set(
-	__isl_take isl_union_set *uset)
+CloogUnionDomain *cloog_union_domain_from_isl_set(
+	__isl_take isl_set *set)
 {
 	int i;
 	int nparam;
-	isl_dim *dim;
+	isl_space *dim;
 	CloogUnionDomain *ud;
 
-	dim = isl_union_set_get_dim(uset);
-	nparam = isl_dim_size(dim, isl_dim_param);
+	dim = isl_set_get_space(set);
+	nparam = isl_space_dim(dim, isl_dim_param);
 
 	ud = cloog_union_domain_alloc(nparam);
 
 	for (i = 0; i < nparam; ++i) {
-		const char *s = isl_dim_get_name(dim, isl_dim_param, i);
+		const char *s = isl_space_get_dim_name(dim, isl_dim_param, i);
 		ud = cloog_union_domain_set_name(ud, CLOOG_PARAM, i, s);
 	}
-	isl_dim_free(dim);
+	isl_space_free(dim);
 
-	if (isl_union_set_foreach_set(uset, &add_domain, &ud) < 0) {
-		isl_union_set_free(uset);
-		cloog_union_domain_free(ud);
-		assert(0);
-	}
-
-	isl_union_set_free(uset);
+	ud = add_domain(set, ud);
 
 	return ud;
 }
@@ -1698,14 +1768,15 @@ struct cloog_can_unroll {
 
 
 /*
- * Check if the given lower bound can be used for unrolling.
+ * Check if the given lower bound can be used for unrolling
+ * and, if so, return the unrolling factor/trip count in *v.
  * If the lower bound involves any existentially quantified
  * variables, we currently punt.
  * Otherwise we compute the maximal value of (i - ceil(l) + 1),
  * with l the given lower bound and i the iterator identified by level.
  */
 static int is_valid_unrolling_lower_bound(struct cloog_can_unroll *ccu,
-	__isl_keep isl_constraint *c)
+	__isl_keep isl_constraint *c, isl_int *v)
 {
 	unsigned n_div;
 	isl_aff *aff;
@@ -1718,8 +1789,8 @@ static int is_valid_unrolling_lower_bound(struct cloog_can_unroll *ccu,
 	aff = isl_constraint_get_bound(c, isl_dim_set, ccu->level - 1);
 	aff = isl_aff_ceil(aff);
 	aff = isl_aff_neg(aff);
-	aff = isl_aff_add_coefficient_si(aff, isl_dim_set, ccu->level - 1, 1);
-	res = isl_set_max(ccu->set, aff, ccu->n);
+	aff = isl_aff_add_coefficient_si(aff, isl_dim_in, ccu->level - 1, 1);
+	res = isl_set_max(ccu->set, aff, v);
 	isl_aff_free(aff);
 
 	if (res == isl_lp_unbounded)
@@ -1727,7 +1798,7 @@ static int is_valid_unrolling_lower_bound(struct cloog_can_unroll *ccu,
 
 	assert(res == isl_lp_ok);
 
-	cloog_int_add_ui(*ccu->n, *ccu->n, 1);
+	cloog_int_add_ui(*v, *v, 1);
 
 	return 1;
 }
@@ -1742,13 +1813,19 @@ static int constraint_can_unroll(__isl_take isl_constraint *c, void *user)
 {
 	struct cloog_can_unroll *ccu = (struct cloog_can_unroll *)user;
 	isl_int v;
+	isl_int count;
 
 	isl_int_init(v);
+	isl_int_init(count);
 	isl_constraint_get_coefficient(c, isl_dim_set, ccu->level - 1, &v);
-	if (isl_int_is_pos(v)) {
-		if (!ccu->c && is_valid_unrolling_lower_bound(ccu, c))
-			ccu->c = isl_constraint_copy(c);
+	if (isl_int_is_pos(v) &&
+	    is_valid_unrolling_lower_bound(ccu, c, &count) &&
+	    (!ccu->c || isl_int_lt(count, *ccu->n))) {
+		isl_constraint_free(ccu->c);
+		ccu->c = isl_constraint_copy(c);
+		isl_int_set(*ccu->n, count);
 	}
+	isl_int_clear(count);
 	isl_int_clear(v);
 	isl_constraint_free(c);
 
@@ -1827,7 +1904,7 @@ CloogDomain *cloog_domain_fixed_offset(CloogDomain *domain,
 	c = cloog_constraint_to_isl(lb);
 	aff = isl_constraint_get_bound(c, isl_dim_set, level - 1);
 	aff = isl_aff_ceil(aff);
-	aff = isl_aff_add_coefficient_si(aff, isl_dim_set, level - 1, -1);
+	aff = isl_aff_add_coefficient_si(aff, isl_dim_in, level - 1, -1);
 	aff = isl_aff_add_constant(aff, offset);
 	eq = isl_equality_from_aff(aff);
 	set = isl_set_add_constraint(set, eq);
