@@ -247,6 +247,7 @@ int domain, block, inner, next ;
     if ((inner) && (loop->inner != NULL))
     cloog_loop_free_parts(loop->inner,domain,block,inner,1) ;
     
+    cloog_domain_free(loop->unsimplified);
     cloog_stride_free(loop->stride);
     free(loop) ;
     if (next)
@@ -1596,28 +1597,96 @@ int cloog_loop_is_constant(CloogLoop *loop, int level)
 CloogLoop *cloog_loop_constant(CloogLoop *loop, int level)
 {
     CloogLoop *res, *inner, *tmp;
-    CloogDomain *domain, *context, *t;
+    CloogDomain *domain, *t;
 
     if (!loop)
 	return loop;
 
     inner = loop->inner;
-    for (tmp = loop->next; tmp; tmp = tmp->next)
+    domain = loop->domain;
+    for (tmp = loop->next; tmp; tmp = tmp->next) {
 	inner = cloog_loop_concat(inner, tmp->inner);
+	domain = cloog_domain_union(domain, tmp->domain);
+    }
 
-    domain = cloog_domain_copy(loop->domain);
     domain = cloog_domain_simple_convex(t = domain);
     cloog_domain_free(t);
-    context = cloog_domain_project(domain, level - 1);
-    context = cloog_domain_extend(t = context, level);
-    cloog_domain_free(t);
-    domain = cloog_domain_simplify(t = domain, context);
-    cloog_domain_free(t);
-    cloog_domain_free(context);
 
     res = cloog_loop_alloc(loop->state, domain, 0, NULL, NULL, inner, NULL);
 
-    cloog_loop_free_parts(loop, 1, 0, 0, 1);
+    cloog_loop_free_parts(loop, 0, 0, 0, 1);
+
+    return res;
+}
+
+
+/* Unroll the given loop at the given level, provided it is allowed
+ * by cloog_domain_can_unroll.
+ * If so, we return a list of loops, one for each iteration of the original
+ * loop.  Otherwise, we simply return the original loop.
+ */
+static CloogLoop *loop_unroll(CloogLoop *loop, int level)
+{
+    int can_unroll;
+    cloog_int_t i;
+    cloog_int_t n;
+    CloogConstraint *lb;
+    CloogLoop *res = NULL;
+    CloogLoop **next_res = &res;
+    CloogDomain *domain;
+    CloogLoop *inner;
+
+    cloog_int_init(n);
+    can_unroll = cloog_domain_can_unroll(loop->domain, level, &n, &lb);
+    if (!can_unroll) {
+	cloog_int_clear(n);
+	return loop;
+    }
+
+    cloog_int_init(i);
+
+    for (cloog_int_set_si(i, 0); cloog_int_lt(i, n); cloog_int_add_ui(i, i, 1)) {
+	domain = cloog_domain_copy(loop->domain);
+	domain = cloog_domain_fixed_offset(domain, level, lb, i);
+	inner = cloog_loop_copy(loop->inner);
+	inner = cloog_loop_restrict_all(inner, domain);
+	if (!inner) {
+	    cloog_domain_free(domain);
+	    continue;
+	}
+	*next_res = cloog_loop_alloc(loop->state, domain, 1, NULL, NULL,
+					inner, NULL);
+	next_res = &(*next_res)->next;
+    }
+
+    cloog_int_clear(i);
+    cloog_int_clear(n);
+    cloog_constraint_release(lb);
+
+    cloog_loop_free(loop);
+
+    return res;
+}
+
+
+/* Unroll all loops in the given list at the given level, provided
+ * they can be unrolled.
+ */
+CloogLoop *cloog_loop_unroll(CloogLoop *loop, int level)
+{
+    CloogLoop *now, *next;
+    CloogLoop *res = NULL;
+    CloogLoop **next_res = &res;
+
+    for (now = loop; now; now = next) {
+	next = now->next;
+	now->next = NULL;
+
+	*next_res = loop_unroll(now, level);
+
+	while (*next_res)
+	    next_res = &(*next_res)->next;
+    }
 
     return res;
 }
@@ -1627,6 +1696,110 @@ CloogLoop *cloog_loop_generate_restricted_or_stop(CloogLoop *loop,
 	int level, int scalar, int *scaldims, int nb_scattdims,
 	CloogOptions *options);
 
+CloogLoop *cloog_loop_recurse(CloogLoop *loop,
+	int level, int scalar, int *scaldims, int nb_scattdims,
+	int constant, CloogOptions *options);
+
+
+/**
+ * Recurse on the inner loops of the given single loop.
+ *
+ * - loop is the loop for which we have to generate scanning code,
+ * - level is the current non-scalar dimension,
+ * - scalar is the current scalar dimension,
+ * - scaldims is the boolean array saying whether a dimension is scalar or not,
+ * - nb_scattdims is the size of the scaldims array,
+ * - constant is true if the loop is known to be executed at most once
+ * - options are the general code generation options.
+ */
+static CloogLoop *loop_recurse(CloogLoop *loop,
+	int level, int scalar, int *scaldims, int nb_scattdims,
+	int constant, CloogOptions *options)
+{
+    CloogLoop *inner, *into, *end, *next, *l, *now;
+    CloogDomain *domain;
+
+    if (level && options->strides && !constant)
+      cloog_loop_stride(loop, level);
+
+    if (!constant &&
+	options->first_unroll >= 0 && level + scalar >= options->first_unroll) {
+	loop = cloog_loop_unroll(loop, level);
+	if (loop->next)
+	    return cloog_loop_recurse(loop, level, scalar, scaldims,
+					nb_scattdims, 1, options);
+    }
+
+    if (level && options->otl)
+      cloog_loop_otl(loop, level);
+    inner = loop->inner;
+    domain = cloog_domain_copy(loop->domain);
+    domain = cloog_domain_add_stride_constraint(domain, loop->stride);
+    into = NULL ;
+    while (inner != NULL)
+    { /* 4b. -ced- recurse for each sub-list of non terminal loops. */
+      if (cloog_loop_more(inner, level + 1, scalar, nb_scattdims)) {
+	end = inner;
+        while ((end->next != NULL) &&
+               cloog_loop_more(end->next, level + 1, scalar, nb_scattdims))
+        end = end->next ;
+        
+	next = end->next ;
+        end->next = NULL ;
+
+        l = cloog_loop_generate_restricted_or_stop(inner, domain,
+			level + 1, scalar, scaldims, nb_scattdims, options);
+        
+	if (l != NULL)
+        cloog_loop_add_list(&into,&now,l) ;
+        
+        inner = next ;
+      }
+      else
+      { cloog_loop_add(&into,&now,inner) ;
+        inner = inner->next ;
+      }
+    }
+
+    cloog_domain_free(domain);
+    loop->inner = into;
+    return loop;
+}
+
+
+/**
+ * Recurse on the inner loops of each of the loops in the loop list.
+ *
+ * - loop is the loop list for which we have to generate scanning code,
+ * - level is the current non-scalar dimension,
+ * - scalar is the current scalar dimension,
+ * - scaldims is the boolean array saying whether a dimension is scalar or not,
+ * - nb_scattdims is the size of the scaldims array,
+ * - constant is true if the loop is known to be executed at most once
+ * - options are the general code generation options.
+ */
+CloogLoop *cloog_loop_recurse(CloogLoop *loop,
+	int level, int scalar, int *scaldims, int nb_scattdims,
+	int constant, CloogOptions *options)
+{
+    CloogLoop *now, *next;
+    CloogLoop *res = NULL;
+    CloogLoop **next_res = &res;
+
+    for (now = loop; now; now = next) {
+	next = now->next;
+	now->next = NULL;
+
+	*next_res = loop_recurse(now, level, scalar, scaldims, nb_scattdims,
+				 constant, options);
+
+	while (*next_res)
+	    next_res = &(*next_res)->next;
+    }
+
+    return res;
+}
+
 /**
  * cloog_loop_generate_general function:
  * Adaptation from LoopGen 0.4 by F. Quillere. This function implements the
@@ -1654,15 +1827,15 @@ CloogLoop *cloog_loop_generate_general(CloogLoop *loop,
 	int level, int scalar, int *scaldims, int nb_scattdims,
 	CloogOptions *options)
 {
-  CloogLoop * res, * now, * temp, * l, * new_loop, * inner, * now2, * end,
-            * next, * into ;
-  CloogDomain * domain ;
+  CloogLoop *res, *now, *temp, *l, *new_loop, *next;
   int separate = 0;
+  int constant = 0;
 
   /* 3. Separate all projections into disjoint polyhedra. */
-  if (level > 0 && cloog_loop_is_constant(loop, level))
+  if (level > 0 && cloog_loop_is_constant(loop, level)) {
     res = cloog_loop_constant(loop, level);
-  else if ((options->f > level+scalar) || (options->f < 0))
+    constant = 1;
+  } else if ((options->f > level+scalar) || (options->f < 0))
     res = cloog_loop_merge(loop, level, options);
   else {
     res = cloog_loop_separate(loop);
@@ -1681,44 +1854,8 @@ CloogLoop *cloog_loop_generate_general(CloogLoop *loop,
   temp = res ;
   res = NULL ;
   if (!level || (level+scalar < options->l) || (options->l < 0))
-  while(temp != NULL)
-  { if (level && options->strides)
-    cloog_loop_stride(temp, level);
-    if (level && options->otl)
-      cloog_loop_otl(temp, level);
-    inner = temp->inner ;
-    domain = temp->domain ;
-    into = NULL ;
-    while (inner != NULL)
-    { /* 4b. -ced- recurse for each sub-list of non terminal loops. */
-      if (cloog_loop_more(inner, level + 1, scalar, nb_scattdims)) {
-	end = inner;
-        while ((end->next != NULL) &&
-               cloog_loop_more(end->next, level + 1, scalar, nb_scattdims))
-        end = end->next ;
-        
-	next = end->next ;
-        end->next = NULL ;
-
-        l = cloog_loop_generate_restricted_or_stop(inner, domain,
-			level + 1, scalar, scaldims, nb_scattdims, options);
-        
-	if (l != NULL)
-        cloog_loop_add_list(&into,&now,l) ;
-        
-        inner = next ;
-      }
-      else
-      { cloog_loop_add(&into,&now,inner) ;
-        inner = inner->next ;
-      }
-    }
-    next = temp->next ;
-    temp->next = NULL ;
-    temp->inner = into ;
-    cloog_loop_add(&res,&now2,temp) ;
-    temp = next ;
-  }
+    res = cloog_loop_recurse(temp, level, scalar, scaldims, nb_scattdims,
+			     constant, options);
   else
   while (temp != NULL)
   { next = temp->next ;
@@ -2412,9 +2549,6 @@ static CloogLoop *loop_simplify(CloogLoop *loop, CloogDomain *context,
   CloogLoop *simplified, *inner;
   CloogDomain * domain, * simp, * inter, * extended_context ;
 
-  if (!cloog_domain_isconvex(loop->domain))
-    loop->domain = cloog_domain_simplify_union(loop->domain);
-
   domain = loop->domain ;
   
   domain_dim = cloog_domain_dimension(domain);
@@ -2445,10 +2579,16 @@ static CloogLoop *loop_simplify(CloogLoop *loop, CloogDomain *context,
   simplified = cloog_loop_alloc(loop->state, simp, loop->otl, loop->stride,
 				new_block, inner, NULL);
 
-  /* Only save the domains, if their level is still a scattering level.  */
-  if (options->save_domains && level <= nb_scattdims)
+  /* Only save the domains, if it involves only scattering dimensions.  */
+  if (options->save_domains) {
+    if (domain_dim > nb_scattdims) {
+      CloogDomain *t;
+      inter = cloog_domain_project(t = inter, nb_scattdims);
+      cloog_domain_free(t);
+    }
+    inter = cloog_domain_add_stride_constraint(inter, loop->stride);
     simplified->unsimplified = inter;
-  else
+  } else
     cloog_domain_free(inter);
 
   return(simplified) ; 
@@ -2482,6 +2622,21 @@ CloogLoop *cloog_loop_simplify(CloogLoop *loop, CloogDomain *context, int level,
   CloogLoop *now;
   CloogLoop *res = NULL;
   CloogLoop **next = &res;
+  int need_split = 0;
+
+  for (now = loop; now; now = now->next)
+    if (!cloog_domain_isconvex(now->domain)) {
+      now->domain = cloog_domain_simplify_union(now->domain);
+      if (!cloog_domain_isconvex(now->domain))
+	need_split = 1;
+    }
+  
+  /* If the input of CLooG contains any union domains, then they
+   * may not have been split yet at this point.  Do so now as the
+   * clast construction assumes there are no union domains.
+   */
+  if (need_split)
+    loop = cloog_loop_disjoint(loop);
 
   for (now = loop; now; now = now->next) {
     *next = loop_simplify(now, context, level, nb_scattdims, options);
@@ -2494,12 +2649,6 @@ CloogLoop *cloog_loop_simplify(CloogLoop *loop, CloogDomain *context, int level,
       next = &(*next)->next;
   }
   cloog_loop_free(loop);
-  
-  /* Examples like test/iftest2.cloog give unions of polyhedra after
-   * simplifying, thus we have to make them disjoint. Another good reason to
-   * put the simplifying step in the Quillere backtrack.
-   */
-  res = cloog_loop_disjoint(res);
 
   return res;
 }
